LocalNodesAnalysis=function(path,data_separ,masking,mask_path,mask_separ,regress_separ,decomposition_separ,nodeMeasureName, binarize, weighted, decomposition_method, automatic_decomposition, decomposition_file, alpha=0.05, alpha2=0.05, strong=2, alternative="two.sided", procedure="bonferroni", testType = 'parametric', plotResults=FALSE, regressors_file=NULL,regress = FALSE,paired=FALSE){

if(weighted == FALSE){
weighted = NULL}

library(igraph)
source(paste(connectomeAnalyzerPath,'loadData.r',sep=""))
temp = loadData(path,separ=data_separ,masking,mask_path,mask_separ)

########## Removes multivariate dimension of data
data = temp
data$dataGroup1 = temp$dataGroup1[,,,1]
data$dataGroup2 = temp$dataGroup2[,,,1]

if(decomposition_method == 'File')
{source(paste(connectomeAnalyzerPath,"load_subregions.r", sep=""))
SubregionsIndices=loadSubregions(decomposition_file,decomposition_separ)}
else
{
	if(decomposition_method == 'Automatic')
	{
		source(paste(connectomeAnalyzerPath,"decomposition.r", sep=""))
		SubregionsIndices=decomposition(data,automatic_decomposition)
	}
	else
	{print('Unknown decomposition method')}
}
source(paste(connectomeAnalyzerPath,"SummaryFunctions.r", sep=""))
source(paste(connectomeAnalyzerPath,"ConstructVectorFilter.r", sep=""))
source(paste(connectomeAnalyzerPath,"ComputeK.r", sep=""))
source(paste(connectomeAnalyzerPath,"computePvalue.r", sep=""))
measures=get(nodeMeasureName)
measure=measures
dataGroup1=data$dataGroup1
n1=dim(dataGroup1)[3]
dataGroup2=data$dataGroup2
n2=dim(dataGroup2)[3]
M= dim(dataGroup2)[1]

if(binarize){
	dataGroup1[dataGroup1 > 0 | dataGroup1 < 0] = 1
	dataGroup2[dataGroup2 > 0 | dataGroup2 < 0] = 1
}

group1Measures=array(NA,dim=c(n1, length(measures),dim(dataGroup1)[1]))
group2Measures=array(NA,dim=c(n2, length(measures),dim(dataGroup2)[1]))

for(k in 1:n1){
Graph <- graph.adjacency(dataGroup1[,,k], mode="undirected", weighted=weighted)
group1Measures[k,1,]=measure(Graph)
}

for(k in 1:n2){
Graph <- graph.adjacency(dataGroup2[,,k], mode="undirected", weighted=weighted)
group2Measures[k,1,]=measure(Graph)
}

regressors = NULL
if(regress){
	source(paste(connectomeAnalyzerPath,"load_regressors.r",sep=""))
	regressors = loadRegressors(regressors_file,regress_separ,data$names)	
	for (i in 1:M){ # for each node
		vect=c(group1Measures[, 1, i],group2Measures[, 1, i]) # Vect is a vector with the n1+n2 measures for node i
		if (sum(vect)!=0){
			dat=data.frame(vect, regressors)
			vect_fit=lm(vect~., dat=dat)
			obj = summary(step(vect_fit, trace=0))
			temp= obj$coefficients[1]+ obj$residuals
			group1Measures[,1,i]=temp[1:n1]
			group2Measures[,1,i]=temp[(n1+1):(n1+n2)]		
		}
	}
}
regressors=NULL

p.values=NULL
for (j in 1:dim(dataGroup1)[1]){
p.values=c(p.values,computePvalue(group1Measures[,1,j], group2Measures[,1,j], testType = testType, regressors=NULL,alternative=alternative,paired=paired)$p.value )
}
print(paste("The number of significant results with the AWA for", nodeMeasureName,":"))
print(sum(p.adjust(p.values, procedure)<=alpha2))


reduceVectorData=function(data, summaryFunction, SubregionsIndices){
# n is the number of subjects
n=dim(data)[1]
# s is the number of subsets
sm=length(unique(SubregionsIndices[,1]))
reducedData=matrix(0,n,sm)
for (ir in 1:sm){    # ir is the row index of the brain subregion i,
rowIndex=as.vector(SubregionsIndices[,2][SubregionsIndices[,1]==ir])
for(k in 1:n){
reducedData[k,ir]=summaryFunction(as.vector(data[k,rowIndex]))
}
}
return(reducedData)
}

reducedgroup1=reduceVectorData(group1Measures[,1,], mean, SubregionsIndices)

reducedgroup2=reduceVectorData(group2Measures[,1,], mean, SubregionsIndices)

sm= length(unique(SubregionsIndices[,1]))
si=M/sm
P.values=NULL
for (i in 1:sm){
P.values=c(P.values,computePvalue(reducedgroup1[,i], reducedgroup2[,i], testType = testType, regressors=NULL, alternative=alternative,paired=paired)$p.value)
}



R=sum(p.adjust(P.values, procedure)<=alpha)

ui1= qnorm(1-alpha/(sm))
if (procedure=="BH"){
ui1= qnorm(1-max(1,R)*alpha/(sm))
}
if (procedure=="fdr"){
ui1= qnorm(1-max(1,R)*alpha/(sm))
}

ui2= qnorm(1-alpha)


# compute relaxation coeficient
Scores= qnorm(1-p.values)
relaxation1=computeK2(alpha2, sm, si=si,scoresMatrix=Scores,step=0.05,initK=1,ui1)

VectorOfSignificantSubsets=P.values<=1-pnorm(ui1)
Filter1=ConstructVectorFilter(VectorOfSignificantSubsets, SubregionsIndices)
print(paste("The number of significant results with the RMWC for", nodeMeasureName,":"))
relaxed.p.values=((p.values/relaxation1)*Filter1+(1-Filter1))
print(sum(p.adjust(relaxed.p.values,procedure)<=alpha2))
if (sum(p.adjust(relaxed.p.values,procedure)<=alpha2)>0){
print("The significant locations are")
print(c(1:M)[p.adjust(relaxed.p.values,procedure)<=alpha2])
}

relaxation2=computeK2(alpha2, sm, si=si,scoresMatrix=Scores,step=0.05,initK=1,ui2)
VectorOfSignificantSubsets=P.values<=1-pnorm(ui2)
Filter2=ConstructVectorFilter(VectorOfSignificantSubsets, SubregionsIndices)
print(paste("The number of significant results with the RMNC for", nodeMeasureName,":"))
relaxed.p.values=((p.values/relaxation2)*Filter2+(1-Filter2))
print(sum(p.adjust(relaxed.p.values,procedure)<=alpha2))
if (sum(p.adjust(relaxed.p.values,procedure)<=alpha2)>0){
print(paste("The significant locations are for", nodeMeasureName,":"))
print(c(1:M)[p.adjust(relaxed.p.values,procedure)<=alpha2])
}

relaxation3=computeK3(alpha2, sm, si=si,scoresMatrix=Scores,step=0.05,initK=1,ui1, strong=strong)
VectorOfSignificantSubsets=P.values<=1-pnorm(ui1)
Filter3=ConstructVectorFilter(VectorOfSignificantSubsets, SubregionsIndices)
print(paste("The number of significant results with the RMIO for", nodeMeasureName,":"))
relaxed.p.values=((p.values/relaxation3)*Filter3+(p.values*strong)*(1-Filter3))
print(sum(p.adjust(relaxed.p.values,procedure)<=alpha2))
if (sum(p.adjust(relaxed.p.values,procedure)<=alpha2)>0){
print(paste("The significant locations are for", nodeMeasureName,":"))
print(c(1:M)[p.adjust(relaxed.p.values,procedure)<=alpha2])
}
}

