p.adjust2=function (p, method = c(p.adjust.methods,"gFER1","gFER2","gFDR1","gFDR2"), n = length(p), g=0.5,k=1,beta=10, gamma=0.05){
    method <- match.arg(method)
    if (method == "fdr")
        method <- "BH"
    nm <- names(p)
    p <- as.numeric(p)
    names(p) <- nm
    p0 <- p
    if (all(nna <- !is.na(p)))
        nna <- TRUE
    p <- p[nna]
    lp <- length(p)
    stopifnot(n >= lp)
    if (n <= 1)
        return(p0)
    if (n == 2 && method == "hommel")
        method <- "hochberg"
    p0[nna] <- switch(method, bonferroni = pmin(1, n * p), holm = {
        i <- seq_len(lp)
        o <- order(p)
        ro <- order(o)
        pmin(1, cummax((n - i + 1L) * p[o]))[ro]
    }, gFER1 = {
        i <- seq_len(lp)
        o <- order(p)
        ro <- order(o)
        pmin(1, cummax((n+floor(i^g*gamma)+1-i)/(floor(i^g*gamma)+1) * p[o]))[ro]
    }, gFER2 = {
        i <- seq_len(lp)
        o <- order(p)
        ro <- order(o)
        pmin(1, cummax((n+floor((k-k*exp(-i/beta))*gamma)+1-i)/(floor((k-k*exp(-i/beta))*gamma)+1) * p[o]))[ro]
    }, hommel = {
        if (n > lp) p <- c(p, rep.int(1, n - lp))
        i <- seq_len(n)
        o <- order(p)
        p <- p[o]
        ro <- order(o)
        q <- pa <- rep.int(min(n * p/i), n)
        for (j in (n - 1):2) {
            ij <- seq_len(n - j + 1)
            i2 <- (n - j + 2):n
            q1 <- min(j * p[i2]/(2:j))
            q[ij] <- pmin(j * p[ij], q1)
            q[i2] <- q[n - j + 1]
            pa <- pmax(pa, q)
        }
        pmax(pa, p)[if (lp < n) ro[1:lp] else ro]
    }, hochberg = {
        i <- lp:1L
        o <- order(p, decreasing = TRUE)
        ro <- order(o)
        pmin(1, cummin((n - i + 1L) * p[o]))[ro]
    }, BH = {
        i <- lp:1L
        o <- order(p, decreasing = TRUE)
        ro <- order(o)
        pmin(1, cummin(n/i * p[o]))[ro]
    }, gFDR1 = {
        i <- lp:1L
        o <- order(p, decreasing = TRUE)
        ro <- order(o)
        pmin(1, cummin(n/i^g * p[o]))[ro]
    }, gFDR2 = {
        i <- lp:1L
        o <- order(p, decreasing = TRUE)
        ro <- order(o)
        pmin(1, cummin(n/((k-k*exp(-i/beta))) * p[o]))[ro]
    }, BY = {
        i <- lp:1L
        o <- order(p, decreasing = TRUE)
        ro <- order(o)
        q <- sum(1L/(1L:n))
        pmin(1, cummin(q * n/i * p[o]))[ro]
    }, none = p)
    p0
}

getThreshold=function(MatrixOfPvalues, WieghtsMatrix, alpha=0.05, MCPprocedure="bonferroni", wightedMCP=FALSE, option, g=.5){
# MCPprocedure =  "holm"  or "hochberg" or "hommel" or "bonferroni" "BH" or "BY" or "fdr" or "none" or ""

s=dim(MatrixOfPvalues$MatrixOfOmmission)[1]
if(sum(WieghtsMatrix)!=1){
	temp=NULL
	weights=NULL
	if (option=="one region to one region"){
		for (i in 1:s){
			for(j in 1:i){
				if  (MatrixOfPvalues$MatrixOfOmmission[i,j]==FALSE){
					temp=c(temp, MatrixOfPvalues$MatrixOfPvalues[i,j])
					weights=c(weights, WieghtsMatrix[i,j])
				}
			}
		}
	}

	weights=weights/mean(weights)
	if (MCPprocedure!="pfdr"){
		temp2=p.adjust2(temp/weights, MCPprocedure)
	}
	else{
		temp2=qvalue(temp/weights, pi0.method="bootstrap")$qvalues
	}

	if( min(temp2)<=alpha ){
		threshold=max(temp[temp2<=alpha])
	}
	else{
		threshold=0
	}
	out=NULL
	out$threshold=threshold
	out$minAdjustedPvalue=min(temp2)
	out$numberOfTests=length(temp)
}

else{
	temp=NULL

	if (option=="one region to one region"){
		for (i in 1:s){
			for(j in 1:i){
				if  (MatrixOfPvalues$MatrixOfOmmission[i,j]==FALSE){
					temp=c(temp, MatrixOfPvalues$MatrixOfPvalues[i,j])
				}
			}
		}
	}

	if (option=="one region to all regions"){
		for (i in 1:s){
			if  (MatrixOfPvalues$MatrixOfOmmission[i,1]==FALSE){
				temp=c(temp, MatrixOfPvalues$MatrixOfPvalues[i,1])
			}
		}
	}

	if (MCPprocedure!="pfdr"){
		temp2=p.adjust2(temp, MCPprocedure)
	}
	else{
		temp2=qvalue(temp, pi0.method="bootstrap")$qvalues
	}

	if( min(temp2)<=alpha ){
		threshold=max(temp[temp2<=alpha])
	}
	else{
		threshold=0
	}
	out=NULL
	out$threshold=threshold
	out$minAdjustedPvalue=min(temp2)
	out$numberOfTests=length(temp2)

}

return(out)
}
